﻿using Flurl;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ConsoleApp1
{
    class Program
    {
        public const string Key_AppUniqueName = "getallimages";
        public const string Key_URLProtocol = "URL Protocol";
        public const string Key_DefaultIcon = "DefaultIcon";
        public const string Key_Shell = "shell";
        public const string Key_Open = "open";
        public const string Key_Command = "command";

        static void Main(string[] args)
        {
            if (args.Length > 0 && args[0] == "-r")
            {
                RegisterProtocolHandler(true);
                return;
            }

#if DEBUG
            string url = "getallimages://www.naver.com";
#else
            string url = args[0].ToLower();
#endif
            url = url.Replace(Key_AppUniqueName + "://", "http://");

            DownloadImages(url);
            Console.ReadLine();
        }

        static int _uniqueId;

        private static async void DownloadImages(string url)
        {
            WebClient wc = new WebClient();
            string htmlText = wc.DownloadString(url);

            HtmlAgilityPack.HtmlDocument doc = new HtmlAgilityPack.HtmlDocument();
            doc.LoadHtml(htmlText);

            string localDownloadPath = GetDownloadPath();

            _uniqueId = 0;

            HashSet<string> downloaded = new HashSet<string>();

            foreach (var imgNode in doc.DocumentNode.SelectNodes("//img"))
            {
                string src = imgNode.GetAttributeValue("src", "");

                string srcUrl = UrlCombine(url, src);
                if (downloaded.Contains(srcUrl) == true)
                {
                    continue;
                }

                downloaded.Add(srcUrl);

                await DownloadFile(localDownloadPath, srcUrl);

                Console.WriteLine(srcUrl);
            }
        }

        private static string GetDownloadPath()
        {
            string path = Path.GetDirectoryName(typeof(Program).Assembly.Location);
            path = Path.Combine(path, "images");

            if (Directory.Exists(path) == false)
            {
                Directory.CreateDirectory(path);
            }

            return path;
        }

        private static Task DownloadFile(string localDownloadPath, string srcUrl)
        {
            return Task.Factory.StartNew(
                () =>
                {
                    WebClient wc = new WebClient();

                    Uri uri = new Uri(srcUrl);
                    string fileName = GetUriFileName(uri);

                    string filePath = Path.Combine(localDownloadPath, fileName);

                    wc.DownloadFile(srcUrl, filePath);
                });
        }

        private static string GetUriFileName(Uri uri)
        {
            if (uri.Segments.Length == 0)
            {
                return GetUniqueName();
            }

            string fileName = uri.Segments[uri.Segments.Length - 1];
            string cleanFileName = String.Join("", fileName.Split(Path.GetInvalidFileNameChars()));

            return cleanFileName;
        }

        private static string GetUniqueName()
        {
            int idx = Interlocked.Increment(ref _uniqueId);
            return string.Format("img_{0}", idx);
        }

        public static string UrlCombine(string baseUrl, string url)
        {
            Uri uriBase = new Uri(baseUrl);
            Uri uriRelative = null;

            try
            {
                uriRelative = new Uri(url);

                if (string.IsNullOrEmpty(uriRelative.Scheme) == true)
                {
                    return uriBase.Scheme + "://" + uriRelative.ToString();
                }

                if (uriRelative.OriginalString.StartsWith("//") == true)
                {
                    return uriBase.Scheme + ":" + uriRelative.OriginalString;
                }

                return url;
            }
            catch (Exception)
            {
            }

            return Url.Combine(baseUrl, url);
        }

        static void RegisterProtocolHandler(bool doRegistration)
        {
            string exePath = typeof(Program).Assembly.Location;
            string exeName = Path.GetFileName(exePath);

            if (doRegistration == true)
            {
                using (RegistryKey appKey = Registry.ClassesRoot.CreateSubKey(Key_AppUniqueName, true))
                {
                    appKey.SetValue(null, "URL:Get All Images Protocol");
                    appKey.SetValue(Key_URLProtocol, "");

                    using (RegistryKey defaultIconKey = appKey.CreateSubKey(Key_DefaultIcon, true))
                    {
                        defaultIconKey.SetValue(null, string.Format("{0},1", exeName));
                    }

                    using (RegistryKey shellKey = appKey.CreateSubKey(Key_Shell, true))
                    using (RegistryKey openKey = shellKey.CreateSubKey(Key_Open, true))
                    using (RegistryKey commandKey = openKey.CreateSubKey(Key_Command, true))
                    {
                        string value = string.Format("\"{0}\" \"%1\"", exePath);
                        commandKey.SetValue(null, value);
                    }
                }
            }
        }
    }
}
